package dionarap;

import java.io.File;
import java.util.Vector;

import javax.swing.ImageIcon;

/**
 * Die Klasse <code>Artwork</code> stellt Methoden zum Setzen und
 * Abfragen der Themes und liefert die Icons. Sie ist als Singleton
 * implementiert.
 * 
 * @author Thomas Merkel
 * @author Rainer Hihn
 * @version 1.4
 */
public class Artwork {	
	/** Bezeichnung des Default-Themes */
	static public final String DEFAULT_THEME = "Dracula";
	
	/** Moegliche Erweitungen fuer die Icon-Dateien */
	static private final String[] extensions = new String[] { ".gif", ".jpg", ".png" };
	
	/** Singleton-Instanz */
	static private Artwork artwork;
	
	/** Wurzel-Verzeichnis der Themes */
	private String root;
	
	/** Momentan ausgewaehltes Theme */
	private String themename;
	
	/** Liste der verfuegbaren Themes */
	private Vector<String> themenames;

	/** Gegner, Wurmloch, Hindernis, Spiel-Verloren, Spiel-Gewonnen, Schuss, Munitions Icon */
	private ImageIcon opponent, vortex, obstacle, gameover, gamewon, destruction, ammo;
	
	/** Player-Icons */
	private ImageIcon[] player;
	
	
	/**
	 * Konstruktor der Klasse <code>Artwork</code>. Setzt Wurzel-Verzeichnis und
	 * erstellt die Liste der zur Verfuegung stehenden Themes und setzt
	 * das Default-Theme.
	 */
	private Artwork() {
		this.root = System.getProperty("user.dir") + File.separator + "themes" + File.separator;
		this.themenames = new Vector<String>();
		try {
			File f = new File(this.root);
			File[] files = f.listFiles();
	        for (int i = 0; i < files.length; i++) {
	            if (!files[i].isDirectory()) continue;
	            	
	            this.themenames.add(files[i].getName());
	        }
		} catch (Exception e) {
			/** @todo fangen der Exception, falls Theme nicht gesetzt werden kann */
		}
		
		this.player = new ImageIcon[9];
		
		/**
		 * Setzte default Theme
		 */
		this.setTheme(Artwork.DEFAULT_THEME);
	}
	
	
	/**
	 * Setzt das Theme, liest die Icons neu ein
	 * @param name Theme, das eingestellt werden soll
	 */
	public void setTheme(String name) {
		this.themename = name;
		this.loadIcons();
	}
	
	/**
	 * Liefert die Instanz des Theme-Managers zurueck
	 * @return Instanz von Theme
	 */
	static public Artwork getTheme() {
		if (Artwork.artwork == null) 
			Artwork.artwork = new Artwork();
		
		return Artwork.artwork;
	}
	
	/**
	 * Liest alle fr das Spiel benoetigten Icons ein
	 */
	private void loadIcons() {
 		this.opponent = this.getIcon("opponent");
 		this.obstacle = this.getIcon("obstacle");
 		this.vortex = this.getIcon("vortex");
 		this.gameover = this.getIcon("loss");
 		this.gamewon = this.getIcon("gamewon");
 		this.destruction = this.getIcon("destruction");
 		this.ammo = this.getIcon("ammo");
 				
		for (int i = 0; i < 9; i++) {
			this.player[i] = this.getIcon("player" + (i+1));
		}
	}

	
	/**
	 * Liest ein Icon aus einer Datei. Der Name wird ohne Pfad und ohne Erweiterung
	 * uebergeben, es wird versucht eine passende Datei im aktuellen Theme-Verzeichnis
	 * zu finden.
	 * @param file Dateiname ohne Pfad und ohne Erweiterung
	 * @return gelesenes Icon, Falls keins gefunden <code>null</code>
	 */
	private ImageIcon getIcon(String file)  {
		String basefilename = this.root + this.themename + File.separator + file;
		for (int i = 0; i < Artwork.extensions.length; i++) {
			File test = new File(basefilename + Artwork.extensions[i]);
			if (test.exists()) return new ImageIcon(test.getAbsolutePath());
		}
		System.out.println("Keine passende Datei fuer " + file + " gefunden");
		return null;
	}
	
	
	/**
	 * Liefert die Liste der zur Verfuegung stehenden Themes 
	 * @return Liste als <code>String[]</code>
	 */
	public String[] getThemes() {
		int size = this.themenames.size();
		String[] r = new String[size];
		for (int i = 0; i < size; i++) {
			r[i] = this.themenames.get(i);
		}
		return r;
	}
	
	
	/**
	 * Liefert das aktuell Theme zurueck
	 * @return Theme-Name
	 */
	public String getCurrentTheme() {
		return this.themename;
	}
	
	
	/**
	 * Liefert das Gegner-Icon
	 * @return Gegner-Icon
	 */
	public ImageIcon getOpponentIcon() {
		return this.opponent;
	}

	
	/**
	 * Liefert das Wurmloch-Icon
	 * @return Wurmloch-Icon
	 */
	public ImageIcon getVortexIcon() {
		return this.vortex;
	}
	
	
	/**
	 * Liefert ein Spieler-Icon
	 * @param direction Richtung, in die sich der Spieler bewegt (<code>5</code> = neutral)
	 * @return Spieler-Icon
	 */
	public ImageIcon getPlayerIcon(int direction) {
		return this.player[direction-1];
	}

	
	/**
	 * Liefert das Hinternis-Icon
	 * @return Hinternis-Icon
	 */
	public ImageIcon getObstacleIcon() {
		return this.obstacle;
	}
	
	
	/**
	 * Liefert das Spiel-Verloren-Icon
	 * @return Spiel-Verloren-Icon
	 */
	public ImageIcon getGameOverIcon() {
		return this.gameover;
	}

	
	/**
	 * Liefert das Spiel-Gewonnen-Icon
	 * @return Spiel-Gewonnen-Icon
	 */
	public ImageIcon getGameWonIcon() {
		return this.gamewon;
	}

	
	/**
	 * Liefert das Schuss-Icon
	 * @return Schuss-Icon
	 */
	public ImageIcon getDestructionIcon() {
		return this.destruction;
	}
	
	
	/**
	 * Liefert das Munitions-Icon
	 * @return Munitions-Icon
	 */
	public ImageIcon getAmmoIcon() {
		return this.ammo;
	}

	
}