package dionarap;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import de.dionarap.leveleditor.gui.LevelEditor;
import de.dionarap.leveleditor.model.GameSettingsModel;
import de.fhwgt.dionarap.controller.DionaRapController;
import de.fhwgt.dionarap.levelreader.LevelReader;
import de.fhwgt.dionarap.model.data.DionaRapModel;
import de.fhwgt.dionarap.model.data.MTConfiguration;
import de.fhwgt.dionarap.model.objects.AbstractPawn;

/**
 * Diese Klasse stellt das Hauptfenster von DionaRap zur Verfuegung und enthaelt 
 * die main()-Methode. Diese ist abgeleitet von der Klasse <code>JFrame</code>.
 * 
 * Sie beinhaltet als sichtbare Komponenten die Menueleiste, das Spielfeld
 * und das Navigtor- und das Hilfe-Fenster. Die Spielelogik wird ebenso in dieser
 * Klasse eingebunden.
 * 
 * @see javax.swing.JFrame
 * @see DionaRapController
 * @see DionaRapModel
 * @see MenuBar
 * @see Navigator
 * @see Spielfeld
 *
 * @author Thomas Merkel
 * @author Rainer Hihn
 * @version 1.4
 */
public class Hauptfenster extends JFrame {

	/** Interne Versionsnummer - Serialisieren fuer Applets, Klassentausch */
	private static final long serialVersionUID = 4932458420387758006L;
	
	/** Spiele Titel */
	private static String titel = "DionaRap - TM, RH";
	
	/** DionaRap-Spielfeld */
	private Spielfeld spielfeld;
	
	/** DionaRap-Spiele-Model */
	private DionaRapModel model;
	
	/** DionaRap-Spiele-Controller */
	private DionaRapController controller;
	
	/** Das Hilfefenster vom Typ <code>JDialog</code> */
	private JDialog helpDialog = this.createHelpDialog();	
	
	/** DionaRap-Spiele-Navigator, rechts neben dem Hauptfenster */
	private Navigator navigator = new Navigator(this);
	
	/** Das Spielfiguren-Fenster vom Typ <code>JDialog</code> */
	private JDialog iconDialog = createDescriptionWindow();

	/** LevelEditor vom Typ <code>JDialog</code> */
	private JDialog levelEditorDialog = createLevelEditorWindow();
	
	/** Der Name momentan ausgewaehlten Levels */
	private String levelname;
	
	/** Die Liste der zur Verfuegung stehenden Levels */
	private Vector<String> levelnames;
	
	/** Bezeichnung fuer den Standard-Level */
	public static final String levelDefaultName = "default";

	/** Prefix Level-Dateinamen und Level-Auswahl-ActionCommands */
	public static final String levelPrefix = "level_";


	/**
	 * Konstruktor der Klasse <code>Hauptfenster</code>. Erstellt alle benoetigten
	 * Komponenten wie Menue, Spielfeld und Steuerungsfenster, und initialisiert die
	 * Spiel-Logik. 
	 * 
	 * @param position Postition des Hauptfensters, Angabe in Pixel
	 * @param level Name des Levels
	 */
	public Hauptfenster(Point position, String level) {
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		/** Setzt den Titel des Fensters */
		this.setTitle(titel);
		
		/** Verkleiner- und Vergroesserung des Fensters nicht moeglich */
		this.setResizable(false);
		
		/** BorderLayout als Layout-Manager */
		this.setLayout(new BorderLayout());
		
		/** Rueckt das Fenster an position */
		this.setLocation(position);
		
		this.levelname = level;
		this.addComponentListener(new ListenerFenster(this.navigator));
		this.setupLevels();
		this.readLevel();

		this.model.addModelChangedEventListener(new ListenerModel(this));
		
		this.setJMenuBar(new MenuBar(this, new ListenerMenu(this)));
		
		this.spielfeld = new Spielfeld( this.model.getGrid().getGridSizeX(), this.model.getGrid().getGridSizeY() );
		this.getContentPane().add(this.spielfeld);;
		
		this.updateGame(GameStatus.game);
		
		this.navigator.setNewGameButtonStatus(false);
		this.navigator.setVisible(true);

		/** Zusammenpacken und Anzeigen */
		this.pack();
		this.setVisible(true);
	}
		
	/**
	 * Programmeintrittspunkt von Diona Rap. Erstellt das Hauptfenster 
	 * mit Standardeinstellungen.
	 * 
	 * @see Hauptfenster
	 * @param args Kommandozeilenparameter
	 */
	public static void main(String[] args) {		
		new Hauptfenster(new Point(60, 60), Hauptfenster.levelDefaultName);	
	}

	
	/**
	 * Stellt die get-Methode fr den <code>DionaRapController</code> zur Verfuegung.
	 * 
	 * @see DionaRapController
	 * @return Instanz vom Typ <code>DionaRapController</code>
	 */
	protected DionaRapController getController() {
		return this.controller;
	}
	
	
	/**
	 * Stellt die get-Methode fr den <code>DionaRapModel</code> zur Verfuegung.
	 * 
	 * @see DionaRapModel
	 * @return Instanz vom Typ <code>DionaRapModel</code>
	 */
	protected DionaRapModel getModel() {
		return this.model;
	}
	
	
	/**
	 * Initialisiert die Levelnamen-Liste <code>levelnames</code>. Zuerst wird das Standardlevel
	 * in die Liste eingetragen, die uebrigen Levels werden im Level-Verzeichnis gesucht,
	 * welches ueber <code>getLevelDirectory()</code> ermittelt wird. Level-Dateinamen muessen zwingend
	 * mit <code>levelPrefix</code> beginnen, der eigentliche Level-Name beginnt erst nach diesem
	 * Prefix.
	 */
	private void setupLevels() {	
		this.levelnames = new Vector<String>();
		this.levelnames.add(Hauptfenster.levelDefaultName);
		try {
			File f = new File(this.getDirectory("xml"));
			File[] files = f.listFiles();
	        for (int i = 0; i < files.length; i++) {
	            if (!files[i].isFile()) continue;
	            if (!files[i].getName().startsWith(Hauptfenster.levelPrefix)) continue;
	            
	            String filename = files[i].getName();
	            filename = filename.substring(Hauptfenster.levelPrefix.length(), filename.length() - ".xml".length());
	            
	            if (filename.equalsIgnoreCase(Hauptfenster.levelDefaultName)) {
	            	System.out.println("Reservierter Levelname " + filename);
	            	continue;
	            }
	            
	            this.levelnames.add(filename);
	        }
		} catch (Exception e) {
			/** 
			 * @todo Fehlerbehandlung 
			 */
		}
	}

	
	/**
	 * Liest den aktuellen Level, welcher in <code>levelName</code> steht, ein
	 * und macht ein Multi-Threaded-Spiel daraus. Initialisiert dabei ebenfalls
	 * das <code>DionaRapModel</code>.<br>
	 * Wenn es sich um den Standard-Level handelt, werden die Einstellungen ueber
	 * <code>getDefaultConfiguration</code> festgelegt.
	 * Ansonsten wird der Dateiname ueber <code>LevelFilename</code> gebildet
	 * und die Datei ueber den <code>LevelReader</code> eingelesen.
	 * 
	 * @see DionaRapController
	 * @see DionaRapModel
	 * @see LevelReader
	 * @see MTConfiguration
	 */
	private void readLevel() {	
		if (this.levelname.equals(Hauptfenster.levelDefaultName)) {
			this.model = new DionaRapModel();
			this.controller = new DionaRapController(this.model);
			this.controller.setMultiThreaded(getDefaultConfiguration());
			return;
		}

		System.out.println("Lade Level: " + this.levelname);
		this.model = new DionaRapModel();
		this.controller = new DionaRapController(this.model);
		MTConfiguration mt = new MTConfiguration();
	
		LevelReader lr = new LevelReader(mt, model);
		try {
			lr.readLevel(this.getLevelFilename(this.levelname));
			this.controller.setMultiThreaded(mt);
		} catch (Exception e) {
			System.out.println("Fehler beim Einlesen der XML-Leveldatei: " + e);
		}
	}

	
	/**
	 * Liefert die Standard-Spieleinstellung zurueck.
	 * 
	 * @see MTConfiguration
	 * @return <code>MTConfiguration</code>-Objekt mit Einstellungen
	 */
	private MTConfiguration getDefaultConfiguration() {
		MTConfiguration mt = new MTConfiguration();
		mt.setAlgorithmAStarActive(true);
		mt.setAvoidCollisionWithOpponent(true);
		mt.setAvoidCollisionWithObstacles(true);
		mt.setMinimumTime(800);
		mt.setShotGetsOwnThread(true);
		mt.setOpponentStartWaitTime(2000);
		mt.setOpponentWaitTime(2000);
		mt.setShotWaitTime(500);
		mt.setRandomOpponentWaitTime(false);
		mt.setDynamicOpponentWaitTime(false);
		
		return mt;
	}
	
	
	/**
	 * Bildet aus einem Levelnamen den kompletten Dateinamen samt Pfad.
	 * @param level Level-Name als <code>String</code>
	 * @return Vollstaendiger Dateiname mit Pfad als <code>String</code>
	 */
	private String getLevelFilename(String level) {
		return this.getDirectory("xml") + Hauptfenster.levelPrefix + level + ".xml";
	}

	
	/**
	 * Liefert das Programm-Verzeichnis, nutzt dafuer die Klassen <code>System</code>
	 * und <code>File</code> 
	 * @return Pfad des Programms
	 */
	private String getDirectory(String directory) {
		return System.getProperty("user.dir") + File.separator + directory + File.separator;
	}

	
	/**
	 * Diese Methode wird aufgerufen, um das Spiel zu aktualisieren,
	 * z.B. um das Spielfeld neu zu zeichnen. Dabei werden alle Icons
	 * neu gezeichnet, der Punktestand aktualisiert, und der 
	 * "Neues Spiel"-Button gegebenenfalls aktiviert
	 * @see Spielfeld
	 * @see Navigator
	 * @see GameStatus
     * @param status Information ueber die Aenderungen
	 */
	public void updateGame(GameStatus status) {
		if ((this.model == null) || (this.spielfeld == null)) return;
		
		this.navigator.setPoints(this.model.getScore());
		this.spielfeld.cleanupSpielfeld();
		if (status != GameStatus.game) {
			this.navigator.setNewGameButtonStatus(true);
		}
		this.spielfeld.updateStatus(status);
		AbstractPawn[] pawns = this.model.getAllPawns();
		for (int i = 0; i < pawns.length; i++) {
			this.spielfeld.setPawn(pawns[i]);
		}
	}
	
	/**
	 * Zeigt das LevelEditor-Fenster an
	 * @see Hauptfenster#levelEditorDialog
	 * @see Hauptfenster#createLevelEditorWindow()
	 */
	public void showLevelEditor() {
		this.levelEditorDialog.setVisible(true);
	}
	
	
	/**
	 * Erstellt den Dialog fuer den LevelEditor. Dieser
	 * LevelEditor stammt aus dem vorhanden LevelEditor.jar
	 * @return Erstellter Dialog, unsichtbar
	 */
	private JDialog createLevelEditorWindow() {
		JDialog levDialog;
		levDialog = new JDialog();
		
		JPanel panel = new JPanel();
		panel.add(new LevelEditor(new GameSettingsModel()));
		panel.setVisible(true);

		levDialog.add(panel, BorderLayout.CENTER);
		levDialog.pack();
		
		return levDialog;
	}
	
	
	/**
	 * Zeigt das Spielfiguren-Fenster an
	 * @see Hauptfenster#iconDialog
	 * @see Hauptfenster#createDescriptionWindow()
	 */
	public void showDescriptionWindow() {
		this.iconDialog.setVisible(true);		
	}
	
	
	/**
	 * Erstellt den Dialog fuer die Spiel-Figuren aus dem aktuellen
	 * Icon-Themas. Zur Darstellung der Icons wird die Klasse
	 * <code>MyGraphic</code> verwendet.
	 * @return Erstellter Dialog, unsichtbar
	 */
	private JDialog createDescriptionWindow() {
		Artwork theme = Artwork.getTheme();

		JDialog descDialog;
		descDialog = new JDialog();
		descDialog.setLayout(new BorderLayout());

		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(1,3));

		panel.add(new PaintGraphic("Spieler", theme.getPlayerIcon(5)));
		panel.add(new PaintGraphic("Gegner", theme.getOpponentIcon()));
		panel.add(new PaintGraphic("Hindernis", theme.getObstacleIcon()));
		panel.setVisible(true);
		
		JButton descCloseButton = new JButton("Schlieen");
		descCloseButton.setActionCommand(MenuBar.CMD_CLOSEWINDOW);
		descCloseButton.addActionListener(new ListenerDialog());

		descDialog.add(panel, BorderLayout.CENTER);
		descDialog.add(descCloseButton, BorderLayout.SOUTH);
		
		descDialog.pack();
		
		return descDialog;
	}

	
	/**
	 * Stellt das neue Theme ein und veranlasst das Hauptfenster, das Spielfeld
	 * neu zu zeichnen
	 * @param theme Name des angeklickten Theme
	 * @see Artwork
	 */
	public void setArtwork(String theme) {
		Artwork.getTheme().setTheme(theme);
		
		JDialog tmp = this.createDescriptionWindow();
		tmp.setLocation(this.iconDialog.getLocation());
		tmp.setVisible(this.iconDialog.isVisible());
		this.iconDialog.dispose();
		this.iconDialog = tmp;
		this.requestFocus();
		
		this.updateGame(GameStatus.game);
	}

	
	/**
	 * Legt ein Look-And-Feel fest und aktualisiert dabei das Hauptfenster,
	 * den <code>navigator</code> und den <code>helpDialog</code>.
	 * @see UIManager#setLookAndFeel(String)
	 * @see SwingUtilities#updateComponentTreeUI(java.awt.Component)
	 * @param lookAndFeel Klassenname des Designs
	 */
	public void setLookAndFeel(String lookAndFeel) {
		try {
			UIManager.setLookAndFeel(lookAndFeel);
			SwingUtilities.updateComponentTreeUI(this);
			SwingUtilities.updateComponentTreeUI(this.navigator);
			SwingUtilities.updateComponentTreeUI(this.helpDialog);
			SwingUtilities.updateComponentTreeUI(this.iconDialog);
		} catch (Exception e) {
			System.out.println("setLookAndFeel Exception " + e);
		}

		this.pack();
		this.navigator.pack();
	}

	
	/**
	 * Liefert den Namen des aktuell ausgewaehlten Levels zurueck
	 * @return Level-Name als <code>String</code>
	 */
	public String getCurrentLevel() {
		return this.levelname;
	}

	
	/**
	 * Startet ein neues Spiel mit dem angegeben Level. Dabei werden die aktuellen
	 * Fenster und die Spiellogik zerstoert und ein neues Hauptfenster an alter 
	 * Stelle neu erstellt.
	 * @param level Level-Name, mit dem das neue Spiel begonnen werden soll 
	 */
	public void startNewGame(String level) {
		Point location = this.getLocation();
		
		this.model.clear();
		
		this.navigator.dispose();
		this.iconDialog.dispose();
		this.dispose();
		
		new Hauptfenster(location, level);
	}
	
	
	/**
	 * Erzeugt das Hilfefenster mit der HTML-Beschreibung des Spieles an. Zur Darstellung
	 * der Beschreibung wird eine <code>JScrollPane</code> genutzt. Die Adresse der Hilfe-Datei
	 * wird ueber <code>getHelpFile</code> ermittelt. Der Dialog ist nach Erstellen noch nicht sichtbar.
	 * @return Hilfe-Fenster, unsichtbar
	 */
	private JDialog createHelpDialog() {
		JDialog dlg = new JDialog();
		dlg.setSize(800, 600);
		dlg.setTitle("Spielhilfe");
		dlg.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dlg.setLayout(new BorderLayout());
		
		JEditorPane editorPane = new JEditorPane();
		JScrollPane scrollPane = new JScrollPane(editorPane, 
									    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
									    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		editorPane.setEditable(false);
		
		try {
			editorPane.setPage(getHelpFile());
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
		
		JButton helpCloseButton = new JButton("Schlieen");
		helpCloseButton.setActionCommand(MenuBar.CMD_CLOSEWINDOW);
		helpCloseButton.addActionListener(new ListenerDialog());
		
		dlg.add(scrollPane, BorderLayout.CENTER);
		dlg.add(helpCloseButton, BorderLayout.SOUTH);
		
		return dlg;
	}
	
	/**
	 * Zeigt das Hilfefenster an.
	 * @see Hauptfenster#helpDialog
	 * @see Hauptfenster#createDescriptionWindow()
	 */
	public void showHelpWindow() {
		this.helpDialog.setVisible(true);
	}
	
	
	/**
	 * Liefert das Hilfe-Verzeichnis, nutzt dafuer <code>getDirectory(String directory)</code>
	 * @see Hauptfenster#getDirectory(String)
	 * @return Pfad des Hilfe-Verzeichnisses
	 */
	private String getHelpDirectory() {
		return this.getDirectory("html");
	}
	
	
	/**
	 * Gibt die Adresse vom Typ <code>URL</code> zu der lokalen HTML Datei mit der
	 * Spielbeschreibung zurueck. Der Dateipfad wird in eine <code>URL</code>-Adresse
	 * fuer den <code>JEditorPane</code> umgewandelt.
	 * @see java.net.URL
	 * @return URL zur Hilfedatei in HTML-Format
	 */
	private URL getHelpFile() {
		String helpFilename = this.getHelpDirectory() + "help.html";
		File helpFile = new File(helpFilename);
		URL helpURL = null;
		try {
			helpURL = helpFile.toURI().toURL();
		}
		catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return helpURL;
	}

	
	/**
	 * Liefert die Liste der zur Verfuegung stehenden Levels zurueck. Erstellt
	 * dabei eine Kopie der Liste um die interne Liste vor Aenderungen zu schuetzen.
	 * @return <code>String</code>-Array mit Level-Namen
	 */
	public String[] getLevels() {
		String[] levelarray = new String[this.levelnames.size()];
		for (int i = 0; i < levelarray.length; i++) {
			levelarray[i] = this.levelnames.get(i);
		}

		this.getTitle();
		
		return levelarray;
	}

	
	/**
	 * Wechselt die Sichtbarkeit des Steuerungsfensters. War es sichtbar, wird
	 * es versteckt, war es beim Aufruf unsichtbar, wird es angezeigt.
	 * @see Hauptfenster#navigator
	 */
	public void toggleNavigator() {
		this.navigator.setVisible(!this.navigator.isVisible());
	}
	
}
