package dionarap;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;


/**
 * Diese Klasse realisiert die Menuezeile im Hauptfenster.
 * Es werden die zwei Hauptmenuepunkte erzeugt und deren Untereintraege
 * Die meisten Menueeintruege werden dynamisch erzeugt. 
 * Die Klasse ist abgeleitet von <code>JMenuBar</code>.
 * 
 * @see javax.swing.JMenuBar
 * 
 * @author Thomas Merkel
 * @author Rainer Hihn
 * @version 1.4
 */
public class MenuBar extends JMenuBar {
	/**
	 * Interne Versionsnummer - Serialisieren fuer Applets, Klassentausch
	 */
	private static final long serialVersionUID = 1634318767797551066L;

	/** 
	 * ActionCommand-String fuer Navigationsfenster ein-/ausblenden 
	 */
	static public final String CMD_TOGGLENAVIGATOR = "toggleNavigator";
	
	/**
	 *  ActionCommand-String zum Anzeigen des LevelEditors
	 */
	static public final String CMD_SHOWLEVELEDITOR = "showLevelEditor";
	
	/** 
	 * ActionCommand-String zum Anzeigen der Spielfigurendialogs
	 */
	static public final String CMD_SHOWICONS = "showIcons";
	
	/** 
	 * ActionCommand-String zum Anzeigen des Spielbeschreibung 
	 */
	static public final String CMD_SHOWHELP = "showHelp";
	
	/** 
	 * ActionCommand-String zum Schliessen von Dialogfenstern 
	 */
	static public final String CMD_CLOSEWINDOW = "closeWindow";
	
	/** 
	 * ActionCommand-Prefix-String zum Wechseln des Icon-Themas 
	 */
	static public final String CMD_THEMEPREFIX = "theme_";
	
	/**
	 * Verbindung zum Hauptfenster, wird benoetigt um das Spielfeld neu zu zeichnen
	 * und die Levelnamen zu ermitteln
	 * 
	 * @see Hauptfenster
	 */
	private Hauptfenster parent;

	/**
	 * Verbindung zum Event-Handler, wird lediglich zum Setzen der Listener fuer die
	 * einzelnen Menuepunkte benoetigt.
	 * 
	 * @see ListenerMenu
	 */
	private ListenerMenu listener;
	
	/**
	 * Konstruktor der Klasse <code>MenuBar</code>
	 * Erzeugt die zwei Hauptpunkte "Konfiguration" und "Hilfe" sowie die weiteren Untermenues
	 * 
	 * @see Hauptfenster
	 * @see ListenerMenu
	 * @param parent Elternfenster, welches das Menue beherbergt, vom Typ <code>Hauptfenster</code>
	 * @param listener Menuelistener, der auf die Menue-Ereignisse reagiert, vom Typ <code>ListenerMenu</code> 
	 */
	public MenuBar(Hauptfenster parent, ListenerMenu listener)
	{
		this.parent = parent;
		this.listener = listener;
		
		// Menue: Konfiguration
		JMenu menuConfiguration = new JMenu("Konfigurieren");
		menuConfiguration.setMnemonic('k');
		menuConfiguration.add(this.createMenuItemLookAndFeel());
		menuConfiguration.add(this.createMenuThemes());
		menuConfiguration.add(this.createCheckBoxMenuItem("Navigator", MenuBar.CMD_TOGGLENAVIGATOR, true)); 
		this.add(menuConfiguration);  

		// Menue: Level
		JMenu menuLevel = new JMenu("Level");
		menuLevel.setMnemonic('l');
		menuLevel.add(this.createMenuItem("Level erstellen", MenuBar.CMD_SHOWLEVELEDITOR) );
		menuLevel.add(this.createMenuLevels());
		this.add(menuLevel);
		
		
		// Menue: Hilfe
		JMenu menuHelp = new JMenu("Hilfe");
		menuHelp.setMnemonic('h');
		menuHelp.add(this.createMenuItem("Spielfiguren anzeigen", MenuBar.CMD_SHOWICONS));
		menuHelp.add(this.createMenuItem("Spielbeschreibung", MenuBar.CMD_SHOWHELP));
		this.add(menuHelp);		
	}
	
	
	/**
	 * Hilfsmethode zum Erzeugen des Menuepunkts "Look and Feel". Dieses <code>JMenuItem</code> selbst
	 * hat (dynamisch generierte) Untermenuepunkte vom Typ <code>JRadioButtomMenuItem</code>. Die
	 * Menuepunkte werden mit Hilfe der Klasse <code>UIManager</code> generiert. Sorgt beim Erstellen 
	 * dafuer, dass das eingestellte "Look and Feel" selektiert ist.
	 * 
	 * @see javax.swing.JMenuItem
	 * @see MenuBar#createRadioMenuItem(String, String, ButtonGroup, boolean)
	 * @return <code>JMenuItem</code> "Look and Feel" mit Untermenuepunkten 
	 */
	private JMenuItem createMenuItemLookAndFeel()
	{
		LookAndFeelInfo[] lnf =	UIManager.getInstalledLookAndFeels();

		JMenuItem menuItemLookAndFeel = new JMenu("Look and Feel");

		ButtonGroup groupLookAndFeel = new ButtonGroup();
		String lookandfeel = UIManager.getLookAndFeel().getName();
		for(int i = 0; i < lnf.length; i++) {		
			menuItemLookAndFeel.add(this.createRadioMenuItem(lnf[i].getName(), 
									lnf[i].getClassName(), 
									groupLookAndFeel,
									lookandfeel.equals(lnf[i].getName())));
		}
		
		return menuItemLookAndFeel;
	} 
	
	
	/**
	 * Hilfsmethode zum Erzeugen des Menuepunkts "Theme". Dieses <code>JMenuItem</code> selbst
	 * hat (dynamisch generierte) Untermenuepunkte vom Typ <code>JRadioButtomMenuItem</code>. Die
	 * Menuepunkte werden mit Hilfe der Klasse Artwork generiert. Sorgt beim Erstellen
	 * dafuer, dass das eingestellte Thema selektiert ist.
	 * 
	 * @see javax.swing.JMenuItem
	 * @see MenuBar#createRadioMenuItem(String, String, ButtonGroup, boolean)
	 * @return <code>JMenuItem</code> "Theme" mit Untermenuepunkten 
	 */
	private JMenuItem createMenuThemes()
	{
		String[] themes = Artwork.getTheme().getThemes();
		String currenttheme = Artwork.getTheme().getCurrentTheme();

		JMenuItem menuItemThemes = new JMenu("Thema");		
		
		ButtonGroup groupThemes = new ButtonGroup();
		for (int i = 0; i < themes.length; i++) {
			menuItemThemes.add(this.createRadioMenuItem(themes[i], 
								MenuBar.CMD_THEMEPREFIX + themes[i], 
								groupThemes,
								themes[i].equals(currenttheme)));
		}
		
		return menuItemThemes;
	}
	
	
	/**
	 * Hilfsmethode zum Erzeugen des Menuepunkts "Level laden". Dieses <code>JMenuItem</code> selbst
	 * hat (dynamisch generierte) Untermenpunkte vom Typ <code>JRadioButtomMenuItem</code>. Die
	 * Menuepunkte werden mit Hilfe von <code>getLevels</code> des Hauptfensters generiert.
	 * Sorgt beim Erstellen dafuer, dass das ausgewaehlte Level selektiert ist.
	 * 
	 * @see javax.swing.JMenuItem
	 * @see MenuBar#createRadioMenuItem(String, String, ButtonGroup, boolean)
	 * @return JMenuItem "Level laden" mit Untermenuepunkten 
	 */
	private JMenuItem createMenuLevels()
	{
		String[] levels = this.parent.getLevels();
		String currentLevel = this.parent.getCurrentLevel();

		JMenuItem menuItemLevels = new JMenu("Level laden");
		
		ButtonGroup groupLevels = new ButtonGroup();
		for (int i = 0; i < levels.length; i++) {
			menuItemLevels.add(this.createRadioMenuItem(levels[i], 
								Hauptfenster.levelPrefix + levels[i], 
								groupLevels,
								currentLevel.equals(levels[i])));
		}
		
		return menuItemLevels;
	}
	
	
	/**
	 * Erzeugt ein <code>JMenuItem</code>. Als ActionListener wird der
	 * der dem Konstruktor uebergebene Listener vom Typ <code>ListenerMenu</code> eingestellt.
	 * 
	 * @see javax.swing.JMenuItem
	 * @param title Im Menue erscheinender Text des Menuepunkts
	 * @param command Das geworfene ActionCommand, wenn der Menuepunkt ausgewaehlt wird
	 * @return generiertes <code>JMenuItem</code> 
	 */
	private JMenuItem createMenuItem(String title, String command)
	{
		JMenuItem menuitem = new JMenuItem(title);
		menuitem.addActionListener(this.listener);
		menuitem.setActionCommand(command);
		return menuitem;
	}
	

	/**
	 * Erzeugt ein <code>JRadioButtonMenuItem</code>. Als ActionListener wird der
	 * der dem Konstruktor uebergebene Listener vom Typ <code>ListenerMenu</code> eingestellt.
	 * 
	 * @see javax.swing.JRadioButtonMenuItem
	 * @param title Im Menue erscheinender Text des Menuepunkts
	 * @param command Das geworfene ActionCommand, wenn der Menuepunkt ausgewaehlt wird
	 * @param group Die <code>ButtonGroup</code>, zu der der Menuepunkt gehoert
	 * @param selected Wahrheitswert, welcher <code>true</code> ist, wenn der Menuepunkt selektiert ist, ansonsten <code>false</code>
	 * @return generiertes <code>JRadioButtonMenuItem</code> 
	 */
	private JRadioButtonMenuItem createRadioMenuItem(String title, String command, ButtonGroup group, boolean selected)
	{
		JRadioButtonMenuItem menuitem = new JRadioButtonMenuItem(title);	
		menuitem.setActionCommand(command);
		menuitem.setSelected(selected);
		menuitem.addActionListener(this.listener);
		group.add(menuitem);
		return menuitem;
	}
	

	/**
	 * Erzeugt ein <code>JCheckBoxMenuItem</code>. Als ActionListener wird der
	 * der dem Konstruktor bergebene Listener vom Typ <code>ListenerMenu</code> eingestellt.
	 * 
	 * @see javax.swing.JCheckBoxMenuItem
	 * @param title Im Menue erscheinender Text des Menuepunkts
	 * @param command Das geworfene ActionCommand, wenn der Menuepunkt ausgewaehlt wird
	 * @param selected Wahrheitswert, welcher <code>true</code> ist, wenn der Menuepunkt selektiert ist, ansonsten <code>false</code>
	 * @return generiertes <code>JCheckBoxMenuItem</code> 
	 */
	private JCheckBoxMenuItem createCheckBoxMenuItem(String title, String command, boolean selected)
	{
		JCheckBoxMenuItem menuitem = new JCheckBoxMenuItem(title);
		menuitem.setActionCommand(command);
		menuitem.addActionListener(this.listener);
		menuitem.setState(selected);
		return menuitem;
	}
	
}
